/* -*- c++ -*- */
/* 
 * Copyright 2014 SSL, VNU-UET.
 * 
 * This is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3, or (at your option)
 * any later version.
 * 
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this software; see the file COPYING.  If not, write to
 * the Free Software Foundation, Inc., 51 Franklin Street,
 * Boston, MA 02110-1301, USA.
 */

#ifndef INCLUDED_PNC_SYNC_END_NODE_IMPL_H
#define INCLUDED_PNC_SYNC_END_NODE_IMPL_H

#include <PNC/sync_end_node.h>

namespace gr {
  namespace PNC {

    class sync_end_node_impl : public sync_end_node
    {
     private:
	enum state_t
	{
		ST_IDLE, 			/* for 16 first samples */
		ST_WAIT_STS_SYM,		/* before reaching the first short training symbol*/
		ST_STS_SYM,			/* short training symbols */
		ST_LTS_SYM,			/* long training symbols */
		ST_CHECK_IF_BEACON_OR_DATA,	/* check if this frame contains beacon or data */
		ST_BEACON_DETECTED,		/* this frame contains beacon */
		ST_DATA_DETECTED,		/* this frame contains data */
		ST_SLEEP			/* do nothing */
	};
	state_t d_state;
	int d_window_size; 		  /* size of observation window (samples) */
	int d_window_index;  		  /* index in the observation window */
	int d_xcorr_index; 		  /* index in the cross-correlation window */
	float d_threshold;   		  /* threshold to make decision of sts detected */
	bool d_check_full_xcorr; 	  /* check full cross-correlation window, it's full for 16 samples */
	bool d_check_full_plot;
	int d_shift_count; 		  /* count the number of shifts after the first peak (out of 10 peaks) has been already detected */
	int d_lts_sym_count; 		  /* count the number of received long training symbols */
	int d_beacon_count; 		  /* count the bumber of received beacon samples */
	int d_beacon_index;		  /* beacon index */
	int d_nPeaks_detected; 		  /* number of peaks detected in cross-correlation */
	std::vector<gr_complex> d_window; /* initialize observation window */
	std::vector<double> d_xcorr; 	  /* initialize cross correlation vector */
	std::vector<double> d_xcorr_plot; /* for plotting cross-correlation */
	int d_plot_index;		  /* index of d_xcorr_plot */
	std::vector<int> d_beacon_frame;  /* beacon frame */
	std::vector<gr_complex> d_header; /* header */
	int d_beacon_frame_size;	  /* beacon frame size */
	int d_sts_size; 		  /* size of a short training symbol (802.11a standard, 16 samples) */
	std::vector<gr_complex> d_sts; 	  /* initialize one short training symbol */

     public:
      sync_end_node_impl();
      ~sync_end_node_impl();

      std::vector<double> initialize_xcorr_window(int d_window_size);
      std::vector<gr_complex> initialize_obs_window(int d_window_size);
      std::vector<gr_complex> generate_sts(int d_window_size);
      std::vector<gr_complex> shift_window_func(std::vector<gr_complex> obs_window, gr_complex new_sym, int d_window_size);
      std::vector<double> shift_xcorr_func(std::vector<double> xcorr, double new_xcorr, int d_window_size);

      // Where all the action really happens
      void forecast (int noutput_items, gr_vector_int &ninput_items_required);

      int general_work(int noutput_items,
		       gr_vector_int &ninput_items,
		       gr_vector_const_void_star &input_items,
		       gr_vector_void_star &output_items);
    };

  } // namespace PNC
} // namespace gr

#endif /* INCLUDED_PNC_SYNC_END_NODE_IMPL_H */

